/**
 * \file
 *
 */

#include <netinet/in.h>
#include <netinet/ip6.h>
#include <netinet/icmp6.h>

#ifdef TRACE

/* awfull hack */
extern int DEBUG_indent;
extern int DEBUG_i;
#define return        DEBUG_indent -= 2;return 

#endif /* TRACE */

extern char * global_errmsg;

/* Errors */
/** \brief IPv6 error message */
#define IPV6_IFACESETUP_ERRMSG              "Unable to set up network interface"
/** \brief IPv6 error message */
#define IPV6_IFACESTART_ERRMSG              "Unable to start network interface"
/** \brief IPv6 error message */
#define IPV6_IFACESTOP_ERRMSG               "Unable to stop network interface"
/** \brief IPv6 error message */
#define IPV6_IFACEMTU_ERRMSG                "Unable to set MTU"
/** \brief IPv6 error message */
#define IPV6_IFACEMAC_ERRMSG                "Unable to set MAC address"
/** \brief IPv6 error message */
#define IPV6_IFACEADDR_ERRMSG               "Unable to add IPv6 addresse"
/** \brief IPv6 error message */
#define IPV6_IFACEROUTE_ERRMSG              "Unable to add route"
/** \brief IPv6 error message */
#define IPV6_ROUTERENABLE_ERRMSG            "Unable to switch to router mode"
/** \brief IPv6 error message */
#define IPV6_ROUTERDISABLE_ERRMSG           "Unable to switch to host mode"
/** \brief IPv6 error message */
#define IPV6_CONFIG_INVALIDIFACE_ERRMSG     "Invalid 'iface' section in config file"
/** \brief IPv6 error message */
#define IPV6_CONFIG_INVALIDNODE_ERRMSG      "No node section in config file"
/** \brief IPv6 error message */
#define IPV6_SEND_INVALIDSRC_ERRMSG         "Source address not configured on node"
/** \brief IPv6 error message */
#define IPV6_SEND_INVALIDVERSION_ERRMSG     "Invalid IP version"

/* Error codes (30 -> 49) */
/** \brief IPv6 error code */
#define IPV6_UNKNOWN_ERRCODE                1
/** \brief IPv6 error code */
#define IPV6_EXTERNAL_ERRCODE               2
/** \brief IPv6 error code */
#define IPV6_CONFIG_INVALIDIFACE_ERRCODE    30
/** \brief IPv6 error code */
#define IPV6_CONFIG_INVALIDNODE_ERRCODE     31
/** \brief IPv6 error code */
#define IPV6_SEND_INVALIDSRC_ERRCODE        32
/** \brief IPv6 error code */
#define IPV6_SEND_INVALIDVERSION_ERRCODE    33

/* defs */
/** \brief Default MTU to be used when setting up network interfaces */
#define IPV6_DEFAULT_MTU                    "1500"

/* FreeBSD specific settings */
#ifdef __FreeBSD__

/** \brief ping6 command */
#define IPV6_IFCONFIG_CMD                   "/usr/local/v6/sbin/ifconfig"
#define IPV6_PING6_CMD                      "/usr/local/v6/sbin/ping6"
#define IPV6_PING6_HLIM_OPT                 "-h"
#define IPV6_IFUP_CMD                       "/usr/local/sbin/stf276_ifstart"
#define IPV6_IFDOWN_CMD                     "/usr/local/sbin/stf276_ifstop"
#define IPV6_IFSETMAC_CMD                   "/usr/local/bin/stf276_set_mac"
#define IPV6_IFRESETMAC_CMD                 "/usr/local/bin/stf276_reset_mac"
#define IPV6_ROUTER_CMD                     "/usr/local/sbin/stf276_router"
#define IPV6_ROUTE_CMD                      "/usr/local/v6/sbin/route"

#if BYTE_ORDER == BIG_ENDIAN
#define IPV6_TCLASS_MASK        0x0ff00000      /* traffic class (8 bits) */
#else
#if BYTE_ORDER == LITTLE_ENDIAN
#define IPV6_TCLASS_MASK        0x0000f00f      /* traffic class (8 bits) */
#endif /* LITTLE_ENDIAN */
#endif /* BIG_ENDIAN */

#endif /* __FreeBSD__ */

#ifdef __linux__

#define IPV6_VERSION         0x60
#define IPV6_VERSION_MASK    0xf0
#if BYTE_ORDER == BIG_ENDIAN
#define IPV6_FLOWLABEL_MASK     0x000fffff      /* flow label (20 bits) */
#define IPV6_TCLASS_MASK        0x0ff00000      /* traffic class (8 bits) */
#else
#if BYTE_ORDER == LITTLE_ENDIAN
#define IPV6_FLOWLABEL_MASK     0xffff0f00      /* flow label (20 bits) */
#define IPV6_TCLASS_MASK        0x0000f00f      /* traffic class (8 bits) */
#endif /* LITTLE_ENDIAN */
#endif /* BIG_ENDIAN */
int      snprintf(char *, size_t, const char *, ...);

/** \brief ping6 command */
#define IPV6_IFCONFIG_CMD                   "/sbin/ifconfig"
#define IPV6_PING6_CMD                      "/bin/ping6"
#define IPV6_PING6_HLIM_OPT                 "-t"
#define IPV6_IFUP_CMD                       "/usr/local/sbin/stf276_ifstart"
#define IPV6_IFDOWN_CMD                     "/usr/local/sbin/stf276_ifstop"
#define IPV6_IFSETMAC_CMD                   "/usr/local/bin/stf276_set_mac"
#define IPV6_IFRESETMAC_CMD                 "/usr/local/bin/stf276_reset_mac"
#define IPV6_ROUTER_CMD                     "/usr/local/sbin/stf276_router"
#define IPV6_ROUTE_CMD                      "/sbin/route"
#endif /* __linux__ */


/**
 * \fn      int ipv6IfaceStop(char *iface)
 * \brief   Disable a network interface
 *
 * \param   iface Network interface to disable
 *
 * \return  0 if successful
 * \return  1+ if error occurs
 */
int ipv6IfaceStop(char *iface);

/**
 * \fn      int ipv6IfaceStart(char *iface)
 * \brief   Enable a network interface
 *
 * \param   iface Network interface to enable
 *
 * \return  0 if successful
 * \return  1+ if error occurs
 */
int ipv6IfaceStart(char *iface);

/**
 * \fn      int ipv6IfaceSetMtu(char *iface, char *mtu)
 * \brief   Set the Maximum Transfert Unit of a network interface
 *
 * \param   iface Network interface which will be modified
 * \param   mtu MTU value
 *
 * \return  0 if successful
 * \return  1+ if error occurs
 */
int ipv6IfaceSetMtu(char *iface, char *mtu);

/**
 * \fn      int ipv6IfaceSetMac(char *iface, char *mac)
 * \brief   Set the MAC address of a network interface
 *
 * \param   iface Network interface which will be modified
 * \param   mac MAC address
 *
 * \return  0 if successful
 * \return  1+ if error occurs
 */
int ipv6IfaceSetMac(char *iface, char *mac);

/**
 * \fn      int ipv6IfaceSetAddr(char *iface, char *addr)
 * \brief   Set an IPv6 address for a network interface
 *
 * \param   iface Network interface which will be modified
 * \param   addr IPv6 address
 *
 * \return  0 if successful
 * \return  1+ if error occurs
 */
int ipv6IfaceSetAddr(char *iface, char *addr);

/**
 * \fn      int ipv6NodeEnableRouter()
 * \brief   Switch the node to 'Router Mode'
 *
 * \return  0 if successful
 * \return  1+ if error occurs
 */
int ipv6NodeEnableRouter();

/**
 * \fn      int ipv6NodeDisableRouter()
 * \brief   Switch the node to 'Host Mode'
 *
 * \return  0 if successful
 * \return  1+ if error occurs
 */
int ipv6NodeDisableRouter();

/**
 * \fn      int ipv6IfaceSetRoute(char *dest, char *next);
 * \brief   Set a route
 *
 * \param   dest network or host the route is set for
 * \param   next router to use
 *
 * \return  0 if successful
 * \return  1+ if error occurs
 */
int ipv6IfaceSetRoute(char *dest, char *next);

/**
 * \fn      int ipv6FlushRoutes()
 * \brief   Flush previously set routes
 *
 * \return  0 if successful
 * \return  1+ if error occurs
 */
int ipv6FlushRoutes();

/**
 * \fn      int ipv6SendRaw(struct ip6_hdr *header, int nb_hop, struct in6_addr *hops, unsigned char *data)
 * \brief   Send an IPv6 packet
 *
 * \param   header IPv6 header data
 * \param   nb_hop Number of hops (Routing Header)
 * \param   hops List of hops (Routing Header) 
 * \param   data Payload
 *
 * \return  0 if successful
 * \return  1+ if error occurs
 */
int ipv6SendRaw(struct ip6_hdr *header, int nb_hop, struct in6_addr *hops, unsigned char *data);

/**
 * \fn      int ipv6SendPing(struct ip6_hdr *header, int nb_hop, struct in6_addr *hops)
 * \brief   Send an ICMPv6 Echo Request packet using ping6
 *
 * \param   header IPv6 header data
 * \param   nb_hop Number of hops (Routing Header)
 * \param   hops List of hops (Routing Header) 
 *
 * \return  0 if successful
 * \return  1+ if error occurs
 */
int ipv6SendPing(struct ip6_hdr *header, int nb_hop, struct in6_addr *hops);

/**
 * \fn      int ipv6Decode(unsigned char *data)
 * \brief   Decode IPv6 headers
 *
 * \param   data IPv6 packet
 *
 * \return  0 if successful
 * \return  1+ if error occurs
 */
int ipv6Decode(unsigned char *data);

/**
 * \fn      int ipv6Get(unsigned char **result)
 * \brief   Retieve an IPv6 packet 
 *
 * \param   result address of the buffer where the packet will be stored
 *
 * \return  0 if successful
 * \return  1+ if error occurs
 */
int ipv6Get(char **result);

