/**
 * \file
 * \brief SCOP Encode module (SCOP Interface)
 */


#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <err.h>
#include <errno.h>

#include <math.h>

#include "scop.h"

void scopEncodeInt(int value, char **result)
{
#ifdef TRACE
  DEBUG_indent += 2;
  for(DEBUG_i=0; DEBUG_i<(DEBUG_indent-2); DEBUG_i+=2, printf("| "));
  printf("|-TRACE: %s()\n",__FUNCTION__);
#endif

  /* allocate memory */
  (*result) = (char *)malloc(4);
  if((*result) == NULL)
    {
      warnx("%s(): %s", __FUNCTION__, strerror(errno));
      return;
    }

  /* store length */
  (*result)[0] = 2;

  /* trnaslate integer value into ascii */
  snprintf(&((*result)[1]), 3, "%02X", value);
  
  return;
}

void scopEncodeShortString(char *string, char **result)
{
#ifdef TRACE
  DEBUG_indent += 2;
  for(DEBUG_i=0; DEBUG_i<(DEBUG_indent-2); DEBUG_i+=2, printf("| "));
  printf("|-TRACE: %s()\n",__FUNCTION__);
#endif

  /* allocate memory */
  (*result) = (char *)malloc(strlen(string)+2);
  if((*result) == NULL)
    {
      warnx("%s(): %s", __FUNCTION__, strerror(errno));
      return;
    }
  
  /* copy string */
  /* skip the first byte, which will be used to store length */
  strcpy(&((*result)[1]), string);

  /* store length */
  (*result)[0] = strlen(string);
  (*result)[strlen(string)+1] = '\0';
  
  return;
}

void scopEncodeLongString(char *string, char **result)
{
  int i, len, tmp ;

#ifdef TRACE
  DEBUG_indent += 2;
  for(DEBUG_i=0; DEBUG_i<(DEBUG_indent-2); DEBUG_i+=2, printf("| "));
  printf("|-TRACE: %s()\n",__FUNCTION__);
#endif

  len = strlen(string);

  /* allocate memory */
  (*result) = (char *)malloc(len+5);
  if((*result) == NULL)
    {
      warnx("%s(): %s", __FUNCTION__, strerror(errno));
      return;
    }
  
  /* copy string */
  /* skip the four first bytes,
     which will be used to store length */
  strcpy(&((*result)[4]), string);

  /* encode string length */
  for(i=0; i<4; i++)
    {
      tmp = len / pow(256,(3-i));
      len -= tmp * pow(256,(3-i)); 
      (*result)[i] = tmp;
     }

  (*result)[strlen(string)+4] = '\0';
  
  return;
}

int scopResponse(int code, char *desc)
{
  char *enc_type, *enc_code, *enc_desc, *enc_codev, *enc_descv;
  char *msg;
  int msg_len;
 
#ifdef TRACE
  DEBUG_indent += 2;
  for(DEBUG_i=0; DEBUG_i<(DEBUG_indent-2); DEBUG_i+=2, printf("| "));
  printf("|-TRACE: %s()\n",__FUNCTION__);
#endif

  /* encode message type */
  scopEncodeShortString(SCOP_RESPONSE, &enc_type);
  if(enc_type == NULL)
    {
      return SCOP_UNKNOWN_ERRCODE;
    }

  /* encode param name ("code") */
  scopEncodeShortString("code", &enc_code);
  if(enc_code == NULL)
    {
      free(enc_type);
      return SCOP_UNKNOWN_ERRCODE;
    }

  /* encode value */
  scopEncodeInt(code, &enc_codev);
  if(enc_codev == NULL)
    {
      free(enc_type);
      free(enc_code);
      return SCOP_UNKNOWN_ERRCODE;
    }

  /* encode param name ("desc") */
  scopEncodeShortString("desc", &enc_desc);
  if(enc_desc == NULL)
    {
      free(enc_type);
      free(enc_code);
      free(enc_codev);
      return SCOP_UNKNOWN_ERRCODE;
    }
  
  /* encode value */
  scopEncodeShortString(desc, &enc_descv);
  if(enc_descv == NULL)
    {
      free(enc_type);
      free(enc_code);
      free(enc_codev);
      free(enc_desc);
      return SCOP_UNKNOWN_ERRCODE;
    }

  /* compute msg length */
  msg_len = strlen(SCOP_MAGIC) + strlen(SCOP_VERSION) + strlen(enc_type) 
    + strlen(enc_code) + strlen(enc_codev) 
    + strlen(enc_desc) + strlen(enc_descv);

  /* allocate memory */
  msg = (char *)malloc(msg_len + 1);
  if(msg == NULL)
    {
      warnx("%s(): %s", __FUNCTION__, strerror(errno));
      return SCOP_EXTERNAL_ERRCODE;
    }

  /* copy SCOP header */
  strcpy(msg, SCOP_MAGIC);
  strcpy(&(msg[strlen(SCOP_MAGIC)]), SCOP_VERSION);

  /* copy message type */
  strcpy(&(msg[strlen(SCOP_MAGIC)+strlen(SCOP_VERSION)]), enc_type);

  /* copy code param */
  strcpy(&(msg[strlen(SCOP_MAGIC)+strlen(SCOP_VERSION)+strlen(enc_type)]),
	 enc_code);

  /* copy code value */
  strcpy(&(msg[strlen(SCOP_MAGIC)+strlen(SCOP_VERSION)+strlen(enc_type)
	       +strlen(enc_code)]), enc_codev);

  /* copy desc param */
  strcpy(&(msg[strlen(SCOP_MAGIC)+strlen(SCOP_VERSION)+strlen(enc_type)
	       +strlen(enc_code)+strlen(enc_codev)]), enc_desc);

  /* copy desc value */
  strcpy(&(msg[strlen(SCOP_MAGIC)+strlen(SCOP_VERSION)+strlen(enc_type)
	       +strlen(enc_code)+strlen(enc_codev)
	       +strlen(enc_desc)]), enc_descv);

  free(enc_type);
  free(enc_code);
  free(enc_codev);
  free(enc_desc);
  free(enc_descv);

  msg[msg_len]='\0';

  /* send msg to transport layer */
  scopTransportSend(msg, msg_len);

  free(msg);

  return 0;
}

int scopDataResponse(int code, char *data)
{
  char *enc_type, *enc_code, *enc_codev;
  char *enc_data, *enc_datav;
  char *enc_cmd, *enc_cmdv;
  char *msg;
  int msg_len;
 
#ifdef TRACE
  DEBUG_indent += 2;
  for(DEBUG_i=0; DEBUG_i<(DEBUG_indent-2); DEBUG_i+=2, printf("| "));
  printf("|-TRACE: %s()\n",__FUNCTION__);
#endif

  /* encode message type */
  scopEncodeShortString(SCOP_DATA_RESPONSE, &enc_type);
  if(enc_type == NULL)
    {
      return SCOP_UNKNOWN_ERRCODE;
    }

  /* compute msg length */
  msg_len = strlen(SCOP_MAGIC) + strlen(SCOP_VERSION) + strlen(enc_type);

  /* encode param name ("command") */
  scopEncodeShortString("command", &enc_cmd);
  if(enc_cmd == NULL)
    {
      free(enc_type);
      return SCOP_UNKNOWN_ERRCODE;
    }

  /* encode value */
  scopEncodeShortString("sendViaIut", &enc_cmdv);
  if(enc_cmdv == NULL)
    {
      free(enc_type);
      free(enc_cmd);
      return SCOP_UNKNOWN_ERRCODE;
    }

  /* compute msg length */
  msg_len += strlen(enc_cmd) + strlen(enc_cmdv);


  /* encode param name ("code") */
  scopEncodeShortString("code", &enc_code);
  if(enc_code == NULL)
    {
      free(enc_type);
      free(enc_cmd);
      free(enc_cmdv);
      return SCOP_UNKNOWN_ERRCODE;
    }

  /* encode value */
  scopEncodeInt(code, &enc_codev);
  if(enc_codev == NULL)
    {
      free(enc_type);
      free(enc_cmd);
      free(enc_cmdv);
      free(enc_code);
      return SCOP_UNKNOWN_ERRCODE;
    }

  /* compute msg length */
  msg_len += strlen(enc_code) + strlen(enc_codev);

  if(data != NULL)
    {
      
      /* encode param name ("data") */
      scopEncodeShortString("data", &enc_data);
      if(enc_data == NULL)
	{
	  free(enc_type);
	  free(enc_cmd);
	  free(enc_cmdv);
	  free(enc_code);
	  free(enc_codev);
	  return SCOP_UNKNOWN_ERRCODE;
	}
      
      /* encode value*/
      scopEncodeLongString(data, &enc_datav);
      if(enc_datav == NULL)
	{
	  free(enc_type);
	  free(enc_cmd);
	  free(enc_cmdv);
	  free(enc_code);
	  free(enc_codev);
	  free(enc_data);
	  return SCOP_UNKNOWN_ERRCODE;
	}

      /* compute msg length */
      msg_len += strlen(enc_data) + strlen(&enc_datav[4]) + 4;
    }

  /* allocate memory */
  msg = (char *)malloc(msg_len + 1);
  if(msg == NULL)
    {
      warnx("%s(): %s", __FUNCTION__, strerror(errno));
      return SCOP_EXTERNAL_ERRCODE;
    }

  /* copy SCOP header */
  strcpy(msg, SCOP_MAGIC);
  strcpy(&(msg[strlen(SCOP_MAGIC)]), SCOP_VERSION);

  /* copy message type */
  strcpy(&(msg[strlen(SCOP_MAGIC)+strlen(SCOP_VERSION)]), enc_type);

  /* copy cmd param */
  strcpy(&(msg[strlen(SCOP_MAGIC)+strlen(SCOP_VERSION)+strlen(enc_type)]),
	 enc_cmd);

  /* copy cmd value */
  strcpy(&(msg[strlen(SCOP_MAGIC)+strlen(SCOP_VERSION)+strlen(enc_type)
	       +strlen(enc_cmd)]), enc_cmdv);

  /* copy code param */
  strcpy(&(msg[strlen(SCOP_MAGIC)+strlen(SCOP_VERSION)+strlen(enc_type)
	       +strlen(enc_cmd)+strlen(enc_cmdv)]),
	 enc_code);

  /* copy code value */
  strcpy(&(msg[strlen(SCOP_MAGIC)+strlen(SCOP_VERSION)+strlen(enc_type)
	       +strlen(enc_cmd)+strlen(enc_cmdv)
	       +strlen(enc_code)]), enc_codev);

  if(data != NULL)
    {
      /* copy data param */
      memcpy(&(msg[strlen(SCOP_MAGIC)+strlen(SCOP_VERSION)+strlen(enc_type)
		   +strlen(enc_cmd)+strlen(enc_cmdv)
		   +strlen(enc_code)+strlen(enc_codev)]), 
	     enc_data, strlen(enc_data));
      
      /* copy data value */
      memcpy(&(msg[strlen(SCOP_MAGIC)+strlen(SCOP_VERSION)+strlen(enc_type)
		   +strlen(enc_cmd)+strlen(enc_cmdv)
		   +strlen(enc_code)+strlen(enc_codev)
		   +strlen(enc_data)]), 
	     enc_datav, strlen(&enc_datav[4]) + 4);
    }

  free(enc_type);
  free(enc_cmd);
  free(enc_cmdv);
  free(enc_code);
  free(enc_codev);

  if(data != NULL)
    {
      free(enc_data);
      free(enc_datav);
    }

  msg[msg_len]='\0';

  /* send msg to transport layer */
  scopTransportSend(msg, msg_len);

  free(msg);

  return 0;

}
